@@ -60,7 +60,7 @@ static void decodeCIR(char *s, levelTab lt);
 
 /* IMPORTANT NOTE: the number of cartridges is to be confirmed for each type */
 /* Must modify MAX_NUMBER_OF_MODELS each time you touch to existingPrinters */
-#define MAX_NUMBER_OF_MODELS 18
+#define MAX_NUMBER_OF_MODELS 21
 
 struct
 { char *model; /* The model identifier */
@@ -109,6 +109,14 @@ struct
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
                CARTRIDGE_PHOTOCYAN,CARTRIDGE_PHOTOMAGENTA}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"i865",
+     {{"VC",5,{CARTRIDGE_PHOTOBLACK,CARTRIDGE_BLACK,
+               CARTRIDGE_YELLOW,CARTRIDGE_MAGENTA,
+               CARTRIDGE_CYAN}},
+      {"CL",5,{CARTRIDGE_PHOTOBLACK,CARTRIDGE_BLACK,
+               CARTRIDGE_YELLOW,CARTRIDGE_MAGENTA,
+               CARTRIDGE_CYAN}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"i990",
      {{"LS",6,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
@@ -131,7 +139,7 @@ struct
       {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
       {NULL,RESPONSE_INVALID,{}}}},
-    {"iP3000",
+     {"iP3000",
      {{"VC",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
@@ -143,6 +151,12 @@ struct
       {"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"iP3300",
+     {{"VC",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
+      {"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"BJC-6200",
      {{"VC,BK",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                   CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
@@ -151,6 +165,9 @@ struct
      {{"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"MP160",
+     {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"MP360",
      {{"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
@@ -470,13 +487,13 @@ static void decodeCHD(char *s, struct ink_level *level) {
 }
 
 static void decodeCIR(char *s, levelTab lt) {
-  char *keys[8]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C="}, *z;
-  int values[8]={0,0,0,0,0,0,0,0}, i;
+  char *keys[9]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C=",",CL="}, *z;
+  int values[9]={0,0,0,0,0,0,0,0,0}, i;
 
   s+=3;
   s[0]=',';
 
-  for(i=0;i<8;i++) { 
+  for(i=0;i<9;i++) { 
     if( (z=strstr(s,keys[i])) != 0) {
       sscanf(z+strlen(keys[i]),"%d",&values[i]);
     }
@@ -489,16 +506,21 @@ static void decodeCIR(char *s, levelTab lt) {
   lt[CARTRIDGE_YELLOW] = values[5];
   lt[CARTRIDGE_MAGENTA] = values[6];
   lt[CARTRIDGE_CYAN] = values[7];
+  lt[CARTRIDGE_COLOR] = values[8];
 }
 
 static int makeCommand_canon(const char * /* in */ cmd, int cmdLen,
 		      char * /* out */ data) {
   int dataLen = 0;
-  int length,i;
+  int length;
   char cmdHdr[]= "\x1b[K"; /* All commands begin with that, 
                               followed by the request length (2 bytes) */
   char cmdSeparator[] = "\x00\x1e";
 
+#ifdef DEBUG
+  int i;
+#endif
+
   /* b0-b2 : First, command header */
   memcpy(data, cmdHdr, GET_STR_LENGTH(cmdHdr));
   dataLen = GET_STR_LENGTH(cmdHdr);
